package edu.lawrence.androidweather

import android.os.Bundle
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.activity.enableEdgeToEdge
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.text.KeyboardActions
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Settings
import androidx.compose.material3.Button
import androidx.compose.material3.Card
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.HorizontalDivider
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.material3.TextField
import androidx.compose.material3.TopAppBar
import androidx.compose.runtime.Composable
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalFocusManager
import androidx.compose.ui.text.input.ImeAction
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.lifecycle.viewmodel.compose.viewModel
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import edu.lawrence.androidweather.ui.theme.AndroidWeatherTheme
import java.time.Instant
import java.time.Instant.ofEpochSecond
import java.time.LocalDate
import java.time.LocalDateTime
import java.time.ZoneId
import java.time.ZoneOffset
import java.time.format.DateTimeFormatter
import java.time.format.FormatStyle
import java.util.Locale

class MainActivity : ComponentActivity() {
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        LocationPreferences.init(this)
        enableEdgeToEdge()
        setContent {
            AndroidWeatherTheme {
                Scaffold(modifier = Modifier.fillMaxSize()) { innerPadding ->
                    WeatherApp(
                        modifier = Modifier.padding(innerPadding)
                    )
                }
            }
        }
    }
}

@Composable
fun WeatherApp(modifier : Modifier) {
    val vm : WeatherModel = viewModel()
    vm.getForecast()
    val navController = rememberNavController()
    NavHost(navController,startDestination="forecast") {
        composable(route="forecast") {
            WeatherScreen(vm,toOptions = { navController.navigate("location")}, modifier = modifier)
        }
        composable(route="location") {
            LocationScreen(vm,back = { navController.navigate("forecast")}, modifier = modifier)
        }
    }
}

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun WeatherScreen(vm : WeatherModel, toOptions: () -> Unit,modifier: Modifier) {
    Scaffold(topBar = {
        TopAppBar(
            title = {
                Text(
                    "Android Weather",
                )
            },
            actions = {
                IconButton(onClick = toOptions) {
                    Icon(
                        imageVector = Icons.Default.Settings,
                        contentDescription = "Settings"
                    )
                }
            },
        )
    },
        modifier = Modifier.fillMaxSize())
    { innerPadding ->
        ForecastView(vm,modifier.padding(innerPadding))
    }
}

@Composable
fun ForecastView(vm : WeatherModel,modifier:Modifier) {
    val forecast = remember { vm.forecast }
    Column(horizontalAlignment = Alignment.CenterHorizontally, modifier = modifier.fillMaxSize()) {
        Text(vm.location.value.name)
        HorizontalDivider()
        if(forecast.value == null) {
            Text("Loading...")
        } else {
            val days = forecast.value!!.daily
            val offset = forecast.value!!.timezone_offset.toLong()
            Column {
                days.forEach { day ->
                    ForecastDayCard(offset,day)
                }
            }
        }
    }
}

@Composable
fun ForecastDayCard(offset: Long, day : ForecastDay) {
    Card() {
        Row() {
            Column() {
                val time : Instant = ofEpochSecond(day.dt + offset)
                val formatter = DateTimeFormatter.ofLocalizedDate( FormatStyle.LONG )
                    .withLocale(Locale.US)
                    .withZone( ZoneOffset.UTC)
                val date : String = formatter.format(time)
                Text(date)
                Text("High ${day.temp.max} Low ${day.temp.min}")
            }
            Icon(imageVector = nameToIcon(day.weather.get(0).icon), contentDescription = "Weather Icon")
        }
    }
}

@Composable
fun LocationScreen(vm : WeatherModel,back : () -> Unit, modifier : Modifier) {
    val focusManager = LocalFocusManager.current
    val zip = remember { mutableStateOf("") }
    val location = remember { vm.location}
    Column(horizontalAlignment = Alignment.CenterHorizontally, verticalArrangement = Arrangement.Center, modifier = modifier.fillMaxSize()) {
        Row(horizontalArrangement = Arrangement.Start, verticalAlignment = Alignment.Bottom) {
            Text("ZIP Code")
            Spacer(modifier=Modifier.width(10.dp))
            TextField(value = zip.value,
                onValueChange = {newValue : String -> zip.value = newValue},
                keyboardOptions = KeyboardOptions(keyboardType = KeyboardType.Number, imeAction = ImeAction.Done),
                keyboardActions = KeyboardActions(
                    onDone = { focusManager.clearFocus() })
            )
        }
        Spacer(modifier=Modifier.height(20.dp))
        Row() {
            Button(onClick = {vm.setLocationZip(zip.value)}) {
                Text("Search")
            }
        }
        Spacer(modifier=Modifier.height(20.dp))
        Row() {
            Text("Latitude")
            Spacer(modifier = Modifier.width(10.dp))
            Text(location.value.latitude.toString())
        }
        Row() {
            Text("Longitude")
            Spacer(modifier = Modifier.width(10.dp))
            Text(location.value.longitude.toString())
        }
        Row() {
            Text("City")
            Spacer(modifier = Modifier.width(10.dp))
            Text(location.value.name)
        }
        Spacer(modifier=Modifier.height(20.dp))
        Button(onClick = { vm.getForecast()
            back() }) {
            Text("Done")
        }
    }
}
